/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.navigation.ConnectionsTreeToolbarController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.controller.command.connection.AddConnectionFolder',
    'QDesk.controller.command.connection.AddSchedulerConnection',
    'QDesk.controller.command.connection.DeleteConnectionFolder',
    'QDesk.controller.command.connection.DeleteSchedulerConnection',
    'QDesk.controller.command.connection.EditConnectionFolder',
    'QDesk.controller.command.connection.EditSchedulerConnection',
    'QDesk.controller.command.connection.RefreshConnectionFolder',
    'QDesk.controller.command.connection.TestSchedulerConnection',

    'QDesk.enums.scheduler.SchedulerStatus',
    'QDesk.enums.scheduler.SchedulerType',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const'
  ],


  init : function ()
  {
    var me = this;

    // key is a component query, value is a listener configuration
    this.control( {
      '#connectionsTreePanel #refreshButton' : {
        click : me.onRefreshButtonClick
      },
      '#connectionsTreePanel #collapseAllButton' : {
        click : me.onCollapseAllButtonClick
      },
      '#connectionsTreePanel #addFolderButton' : {
        click : me.onAddFolderButtonClick
      },
      '#connectionsTreePanel #addConnectionButton' : {
        click : me.onAddConnectionButtonClick
      },
      '#connectionsTreePanel #deleteFolderOrConnectionButton' : {
        click : me.onDeleteFolderOrConnectionButtonClick
      },
      '#connectionsTreePanel #editFolderOrConnectionButton' : {
        click : me.onEditFolderOrConnectionButtonClick
      },
      '#connectionsTreePanel #testConnectionButton' : {
        click : me.onTestConnectionButtonClick
      },
      '#connectionsTreePanel' : {
        selectionchange : function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length > 0 )  // selection change event is also fired after deselection
          {
            var selectedRecord = selectedRecords[ 0 ];
            var nodeId = selectedRecord.get( 'id' );
            var nodeType = selectedRecord.get( 'nodeType' );
            if ( nodeId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID || nodeType == 'folder' )
            {
              me._onFolderSelected( selectedRecord );
            }
            else if ( nodeType == 'connection' )
            {
              me._onConnectionSelected( selectedRecord );
            }
          }
          else
          {
            me._onUnselected();
          }
        }
      }
    } );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;

    me.callParent( arguments );
  },


  /**
   * Invoked when the refreshButton in the schedulers tree panel's toolbar is clicked.
   * This method reloads the schedulers tree view.
   */
  onRefreshButtonClick : function ()
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {} ).execute();
  },


  /**
   * Invoked when the collapseAllButton in the schedulers tree panel's toolbar is clicked.
   * This method collapses the schedulers tree.
   */
  onCollapseAllButtonClick : function ()
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    treePanel.collapseAll();
  },


  onAddFolderButtonClick : function ()
  {
    var me = this;
    var selectedRecord = me._getSelectedTreeRecord();
    if ( selectedRecord )
    {
      var folderId = selectedRecord.get( 'folderId' );

      Ext.create( 'QDesk.controller.command.connection.AddConnectionFolder', {
        parentFolderId : folderId
      } ).execute();
    }
  },


  onAddConnectionButtonClick : function ()
  {
    var me = this;
    var selectedRecord = me._getSelectedTreeRecord();
    if ( selectedRecord )
    {
      var folderId = selectedRecord.get( 'folderId' );

      Ext.create( 'QDesk.controller.command.connection.AddSchedulerConnection', {
        parentFolderId : folderId,
        schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
      } ).execute();
    }
  },


  onDeleteFolderOrConnectionButtonClick : function ()
  {
    var me = this;
    var selectedRecord = me._getSelectedTreeRecord();
    if ( selectedRecord )
    {
      var nodeType = selectedRecord.get( 'nodeType' );

      if ( nodeType == 'folder' )
      {
        var folderId = selectedRecord.get( 'folderId' );
        if ( folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID )
        {
          // ignore, root node cannot be deleted
        }
        else
        {
          Ext.create( 'QDesk.controller.command.connection.DeleteConnectionFolder', {
            folderId : folderId
          } ).execute();
        }
      }
      else if ( nodeType == 'connection' )
      {
        var connectionId = selectedRecord.get( 'connectionId' );
        Ext.create( 'QDesk.controller.command.connection.DeleteSchedulerConnection', {
          connectionId : connectionId
        } ).execute();
      }
    }
  },


  onEditFolderOrConnectionButtonClick : function ()
  {
    var me = this;
    var selectedRecord = me._getSelectedTreeRecord();
    if ( selectedRecord )
    {
      var nodeType = selectedRecord.get( 'nodeType' );

      if ( nodeType == 'folder' )
      {
        var folderId = selectedRecord.get( 'folderId' );
        if ( folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID )
        {
          // ignore, root node cannot be renamed
        }
        else
        {
          Ext.create( 'QDesk.controller.command.connection.EditConnectionFolder', {
            folderId : folderId,
            folderName : selectedRecord.get( 'name' )
          } ).execute();
        }
      }
      else if ( nodeType == 'connection' )
      {
        var connectionId = selectedRecord.get( 'connectionId' );
        Ext.create( 'QDesk.controller.command.connection.EditSchedulerConnection', {
          connectionId : connectionId,
          schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
        } ).execute();
      }
    }
  },


  onTestConnectionButtonClick : function ()
  {
    var me = this;
    var selectedRecord = me._getSelectedTreeRecord();
    if ( selectedRecord )
    {
      var nodeType = selectedRecord.get( 'nodeType' );

      if ( nodeType == 'folder' )
      {
        // should not happen
        Ext.Error.raise( 'Action is not applicable to folders.' );
      }
      else if ( nodeType == 'connection' )
      {
        var connectionId = selectedRecord.get( 'connectionId' );
        Ext.create( 'QDesk.controller.command.connection.TestSchedulerConnection', {
          connectionId : connectionId,
          schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
        } ).execute();
      }
    }
  },


  /**
   * Invoked when a folder is selected in the connections tree view.
   * @param folderRecord the selected folder.
   */
  _onFolderSelected : function ( folderRecord )
  {
    var me = this;
    var toolBar = me._getToolBar();

    var addConnectionOrFolderButton = Ext.ComponentQuery.query( '#addFolderOrConnectionButton', toolBar )[ 0 ];
    var deleteConnectionOrFolderButton = Ext.ComponentQuery.query( '#deleteFolderOrConnectionButton', toolBar )[ 0 ];
    var editConnectionOrFolderButton = Ext.ComponentQuery.query( '#editFolderOrConnectionButton', toolBar )[ 0 ];

    var testConnectionButton = Ext.ComponentQuery.query( '#testConnectionButton', toolBar )[ 0 ];

    // enable add action
    addConnectionOrFolderButton.enable();

    // enable delete and edit actions; tooltips differ based on whether a folder, or connection is selected
    deleteConnectionOrFolderButton.setTooltip(
        QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.tooltipDeleteFolderButton' ) );
    editConnectionOrFolderButton.setTooltip(
        QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.tooltipEditFolderButton' ) );

    testConnectionButton.disable();

    // delete and edit buttons enabled only for non-root folders
    if ( folderRecord.get( 'folderId' ) == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID )
    {
      deleteConnectionOrFolderButton.disable();
      editConnectionOrFolderButton.disable();
    }
    else
    {
      deleteConnectionOrFolderButton.enable();
      editConnectionOrFolderButton.enable();
    }
  },


  /**
   * Invoked when a connection is selected in the connections tree panel.
   * @param connectionRecord the selected connection.
   */
  _onConnectionSelected : function ( connectionRecord )
  {
    var me = this;
    var toolBar = me._getToolBar();

    var addConnectionOrFolderButton = Ext.ComponentQuery.query( '#addFolderOrConnectionButton', toolBar )[ 0 ];
    var deleteConnectionOrFolderButton = Ext.ComponentQuery.query( '#deleteFolderOrConnectionButton', toolBar )[ 0 ];
    var editConnectionOrFolderButton = Ext.ComponentQuery.query( '#editFolderOrConnectionButton', toolBar )[ 0 ];

    var testConnectionButton = Ext.ComponentQuery.query( '#testConnectionButton', toolBar )[ 0 ];

    // disable add action
    addConnectionOrFolderButton.disable();

    // enable delete and edit actions; tooltips differ based on whether a folder, or connection is selected
    deleteConnectionOrFolderButton.setTooltip(
        QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.tooltipDeleteConnectionButton' ) );
    deleteConnectionOrFolderButton.enable();

    editConnectionOrFolderButton.setTooltip(
        QDesk.Msg.get( 'QDesk.view.navigation.connections.ConnectionsTreePanel.tooltipEditConnectionButton' ) );
    editConnectionOrFolderButton.enable();

    testConnectionButton.enable();
  },


  /**
   * Invoked when a connection, or folder is unselected in the connections tree panel.
   * @private
   */
  _onUnselected : function ()
  {
    var me = this;
    var toolBar = me._getToolBar();

    var addConnectionOrFolderButton = Ext.ComponentQuery.query( '#addFolderOrConnectionButton', toolBar )[ 0 ];
    var deleteConnectionOrFolderButton = Ext.ComponentQuery.query( '#deleteFolderOrConnectionButton', toolBar )[ 0 ];
    var editConnectionOrFolderButton = Ext.ComponentQuery.query( '#editFolderOrConnectionButton', toolBar )[ 0 ];

    var testConnectionButton = Ext.ComponentQuery.query( '#testConnectionButton', toolBar )[ 0 ];

    // disable add action
    addConnectionOrFolderButton.disable();

    // disable delete and edit actions and reset their tooltips (tooltips change dynamically)
    deleteConnectionOrFolderButton.setTooltip( '' );
    deleteConnectionOrFolderButton.disable();

    editConnectionOrFolderButton.setTooltip( '' );
    editConnectionOrFolderButton.disable();

    testConnectionButton.disable();
  },


  /**
   * Returns the currently selected record (a {@link QDesk.model.ConnectionsTreeNode} instance), or
   * null if no record is selected.
   */
  _getSelectedTreeRecord : function ()
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[ 0 ];
    var selectedRecords = treePanel.getSelectionModel().getSelection();
    if ( Ext.isEmpty( selectedRecords ) )
      return null;
    else
      return selectedRecords[ 0 ];
  },


  /**
   * Returns the connections tree panel's toolbar.
   * @return the toolbar.
   */
  _getToolBar : function ()
  {
    var toolBar = Ext.ComponentQuery.query( '#connectionsTreePanel > #toolBar' )[ 0 ];
    if ( !toolBar )
      Ext.Error.raise( 'Connection tree panel toolbar not found.' );

    return toolBar;
  }

} );
