/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


Ext.define( 'QDesk.util.quartz.QuartzEnumUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.QuartzTriggerType',
    'QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction',
    'QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction',
    'QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction',
    'QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction',
    'QDesk.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction',
    'QDesk.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction'

  ],

  singleton : true,

  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzTriggerType value.
   * @param triggerType a QDesk.enums.scheduler.quartz.QuartzTriggerType value.
   * @return the localized repeat interval unit name.
   */
  getTriggerTypeName : function ( triggerType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzTriggerType.' + triggerType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzRepeatIntervalUnit value.
   * @param repeatIntervalUnit a QDesk.enums.scheduler.quartz.QuartzRepeatIntervalUnit value.
   * @return the localized repeat interval unit name.
   */
  getRepeatIntervalUnitName : function ( repeatIntervalUnit )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzRepeatIntervalUnit.' + repeatIntervalUnit );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzIntervalType value.
   * @param intervalType a QDesk.enums.scheduler.quartz.QuartzIntervalType value.
   * @return the localized interval type name.
   */
  getIntervalTypeName : function ( intervalType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzIntervalType.' + intervalType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzTriggerStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.QuartzTriggerStatisticsType value.
   * @return the localized trigger statistics type name.
   */
  getTriggerStatisticsTypeName : function ( statisticsType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzTriggerStatisticsType.' + statisticsType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzJobStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.QuartzJobStatisticsType value.
   * @return the localized job statistics type name.
   */
  getJobStatisticsTypeName : function ( statisticsType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzJobStatisticsType.' + statisticsType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzSchedulerStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.QuartzSchedulerStatisticsType value.
   * @return the localized scheduler statistics type name.
   */
  getSchedulerStatisticsTypeName : function ( statisticsType )
  {
    var args = Array.prototype.slice.call( arguments ).slice( 1 );
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzSchedulerStatisticsType.' + statisticsType, args );
  },


  /**
   * Returns the misfire instruction value for the specified misfire instruction number (int).
   * @param triggerType a trigger type.
   * @param misfireInstructionNumber a misfire instruction number.
   * @returns the trigger type specific misfire instruction name.
   */
  getTriggerMisfireInstructionFromNumber : function ( triggerType, misfireInstructionNumber )
  {
    switch ( triggerType )
    {
      case QDesk.enums.scheduler.quartz.QuartzTriggerType.SIMPLE:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case 0:
            return QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.SMART_POLICY;
          case  1:
            return QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.FIRE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NOW_WITH_EXISTING_REPEAT_COUNT;
          case 3:
            return QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NOW_WITH_REMAINING_REPEAT_COUNT;
          case 4:
            return QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NEXT_WITH_REMAINING_COUNT;
          case 5:
            return QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NEXT_WITH_EXISTING_COUNT;
        }
        break;

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.CRON:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case  0:
            return QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.CALENDAR_INTERVAL:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case 0:
            return QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.DAILY_TIME_INTERVAL:
        switch ( misfireInstructionNumber )
        {
          case -1:
            return QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY;
          case 0:
            return QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.DATE_INTERVAL:
        switch ( misfireInstructionNumber )
        {
          case 0:
            return QDesk.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.NTH_INCLUDED_DAY:
        switch ( misfireInstructionNumber )
        {
          case 0:
            return QDesk.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction.SMART_POLICY;
          case 1:
            return QDesk.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction.FIRE_ONCE_NOW;
          case 2:
            return QDesk.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction.DO_NOTHING;
        }
        break;

      default:
        throw Ext.Error.raise( 'Unexpected trigger type: ' + triggerType );
    }

    Ext.Error.raise( 'Unrecognized misfire instruction number: ' + misfireInstructionNumber +
        ' for trigger type: ' + triggerType );
  },


  /**
   * Returns the localized name for the specified trigger misfire instruction.
   * @param triggerType the trigger type (a QDesk.enums.scheduler.quartz.QuartzTriggerType value).
   * @param misfireInstruction a misfire instruction.
   * @return the localized misfire instruction name.
   */
  getTriggerMisfireInstructionName : function ( triggerType, misfireInstruction )
  {
    switch ( triggerType )
    {
      case QDesk.enums.scheduler.quartz.QuartzTriggerType.SIMPLE:
        return QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.CRON:
        return QDesk.util.quartz.QuartzEnumUtils.getCronTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.CALENDAR_INTERVAL:
        return QDesk.util.quartz.QuartzEnumUtils.getCalendarIntervalTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.DAILY_TIME_INTERVAL:
        return QDesk.util.quartz.QuartzEnumUtils.getDailyTimeIntervalTriggerMisfireInstructionName(
            misfireInstruction );

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.DATE_INTERVAL:
        return QDesk.util.quartz.QuartzEnumUtils.getDateIntervalTriggerMisfireInstructionName( misfireInstruction );

      case QDesk.enums.scheduler.quartz.QuartzTriggerType.NTH_INCLUDED_DAY:
        return QDesk.util.quartz.QuartzEnumUtils.getNthIncludedDayTriggerMisfireInstructionName( misfireInstruction );

      default:
        throw Ext.Error.raise( 'Unexpected trigger type: ' + triggerType );
    }
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction value.
   * @return the localized misfire instruction name.
   */
  getCalendarIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzCalendarIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction value.
   * @return the localized misfire instruction name.
   */
  getCronTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzCronTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction value.
   * @return the localized misfire instruction name.
   */
  getDailyTimeIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzDailyTimeIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction value.
   * @return the localized misfire instruction name.
   */
  getDateIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzDateIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction value.
   * @return the localized misfire instruction name.
   */
  getNthIncludedDayTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction value.
   * @return the localized misfire instruction name.
   */
  getSimpleTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzExecHistoryExportColumn
   * value.
   * @param columnName a QDesk.enums.scheduler.quartz.QuartzExecHistoryExportColumn value.
   * @return the localized column name.
   */
  getExecHistoryExportColumnName : function ( columnName )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzExecHistoryExportColumn.' +
        columnName );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.QuartzMisfiredTriggerRefireStatus
   * value.
   * @param refireStatus a QDesk.enums.scheduler.quartz.QuartzMisfiredTriggerRefireStatus value.
   * @return the localized refire status name.
   */
  getMisfiredTriggerRefireStatusName : function ( refireStatus )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.QuartzMisfiredTriggerRefireStatus.' +
        refireStatus );
  }
} );
