/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Base class for all scheduler views.
 */
Ext.define( 'QDesk.view.base.scheduler.BaseSchedulerView', {
  extend : 'Ext.tab.Panel',

  closable : 'true',

  qdesk : {
    /*
     * Required config parameter with the ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Required config parameter with the node path of the scheduler connection.
     */
    connectionNodePath : undefined,

    /*
     * Required config parameter with QuartzDesk JVM Agent information for the scheduler
     * connection obtained at the time when the scheduler view has been opened.
     */
    jvmAgentInfo : undefined
  },


  /**
   * Returns the connection ID this tab has been created for.
   * @return the connection ID.
   */
  getConnectionId : function ()
  {
    return this.qdesk.connectionId;
  },


  /**
   * Returns the node path of the connection this scheduler view has been opened for.
   * @returns the connection node path.
   */
  getConnectionNodePath : function ()
  {
    return this.qdesk.connectionNodePath;
  },


  /**
   * Sets the node path of the connection this scheduler view has been created for.
   * @param connectionNodePath a connection node path.
   */
  setConnectionNodePath : function ( connectionNodePath )
  {
    this.qdesk.connectionNodePath = connectionNodePath;
  },


  /**
   * Returns the info on the QuartzDesk JVM Agent for the scheduler this tab has been opened for.
   * @return the info object with the following structure:
   * <pre>
   * {
   *   "available" : true,
   *   "compatible" : true
   *   "compatibleVersion: true
   *   "compatibleEdition: true
   *   "version" : {
   *     "major" : 1,
   *     "minor" : 3,
   *     "maintenance" : 0,
   *     "qualifier" : "SNAPSHOT"
   *   },
   *   "edition" : "Enterprise"
   *   "nameAndVersion" : "QuartzDesk JVM Agent v1.3.0-SNAPSHOT",
   *   "displayVersion" : "1.3.0-SNAPSHOT",
   *   "compatibleEdition: true
   *   "configuration" : [
   *     {
   *       "name" : "db.profile",
   *       "value" : "h2"
   *     },
   *     {
   *       "name" : "jmxConnector.jmxmp.enabled",
   *       "value" : "true"
   *     },
   *     ...
   *   ]
   * }
   * </pre>
   */
  getJvmAgentInfo : function ()
  {
    return this.qdesk.jvmAgentInfo;
  },


  /**
   * Returns the scheduler info for the scheduler this tab has been opened for.
   * @return the info object with the following structure:
   * <pre>
   * {
   *   "type" : "QUARTZ"
   *   "version" : {
   *     major : 1,
   *     minor : 2,
   *     maintenance : 3
   *   },
   *
   *   publicApiInfo : {
   *     detected : true/false,
   *     loadedCorrectly : true/false,
   *     compatible : true/false,
   *     ready : true / false,
   *     version : {
   *       major : 1,
   *       minor : 2,
   *       maintenance : 3,
   *       qualifier : foobar
   *     },
   *   }
   * }
   * </pre>
   */
  getSchedulerInfo : function ()
  {
    return this.qdesk.schedulerInfo;
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.connectionNodePath )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionNodePath' );

    if ( config.qdesk.schedulerInfo == null )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( config.qdesk.jvmAgentInfo == null )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    me.callParent( [ config ] );
  }
} );
