/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.message.MessageChannel',
    'QDesk.store.message.MessageChannelProfilesTreeStore',
    'QDesk.util.EnumUtils',
    'QDesk.util.constant.Const'
  ],

  alias : 'widget.MessageChannelProfilesTreePanel',

  cls : 'messageChannelProfilesTreePanel',

  //scrollable : true,

  displayField : 'name',

  rootVisible : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      root : {
        id : QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID,  // ROOT folder ID
        folderId : QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_FOLDER_ID,
        name : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.rootNodeName' ),
        expanded : true,

        /* MessageChannelProfilesTreeNode */
        nodeType : 'folder'
      },

      viewConfig : {
        copy : false,

        plugins : {
          ptype : 'treeviewdragdrop',
          appendOnly : true,
          dragText : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.dragText' )
        }
      },

      store : Ext.create( 'QDesk.store.message.MessageChannelProfilesTreeStore' ),

      dockedItems : [
        {
          xtype : 'toolbar',
          itemId : 'toolBar',
          dock : 'top',
          items : [
            {
              xtype : 'button',
              itemId : 'refreshButton',
              scale : 'small',
              iconCls : 'refresh',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.tooltipRefreshButton' ),
              disabled : true
            },
            {
              xtype : 'button',
              itemId : 'collapseAllButton',
              scale : 'small',
              iconCls : 'collapseAll',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.tooltipCollapseAllButton' ),
              disabled : true
            },
            '-',
            {
              xtype : 'button',
              itemId : 'addFolderOrProfileButton',
              scale : 'small',
              iconCls : 'addFolderOrProfile',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.tooltipAddFolderOrProfileButton' ),
              disabled : true,
              menu : {
                xtype : 'menu',
                ignoreParentClicks : true,
                items : [
                  {
                    itemId : 'addFolderButton',
                    text : QDesk.Msg.get(
                        'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.addFolderButton' ),
                    iconCls : 'addFolder'
                  },
                  '-',
                  {
                    itemId : 'addEmailProfileButton',
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.enums.message.MessageChannel.EMAIL ),
                    iconCls : QDesk.enums.message.MessageChannel.EMAIL
                  },

                  /*
                   * Collaboration message channel profiles.
                   */
                  {
                    text : QDesk.Msg.get(
                        'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.addCollaborationProfileGroup' ),
                    iconCls : 'addCollaborationProfile',
                    menu : [
                      {
                        itemId : 'addCollaborationHipChatProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.COLLABORATION_HIPCHAT ),
                        iconCls : QDesk.enums.message.MessageChannel.COLLABORATION_HIPCHAT
                      },
                      {
                        itemId : 'addCollaborationSlackProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.COLLABORATION_SLACK ),
                        iconCls : QDesk.enums.message.MessageChannel.COLLABORATION_SLACK
                      }
                    ]
                  },

                  /*
                   * Instant message channel profiles.
                   */
                  {
                    text : QDesk.Msg.get(
                        'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.addInstantMessageProfileGroup' ),
                    iconCls : 'addInstantMessageProfile',
                    menu : [
                      {
                        itemId : 'addImAimProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.IM_AIM ),
                        iconCls : QDesk.enums.message.MessageChannel.IM_AIM
                      },
                      {
                        itemId : 'addImGTalkProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.IM_GTALK ),
                        iconCls : QDesk.enums.message.MessageChannel.IM_GTALK
                      },
                      {
                        itemId : 'addImIcqProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.IM_ICQ ),
                        iconCls : QDesk.enums.message.MessageChannel.IM_ICQ
                      },
                      {
                        itemId : 'addImXmppProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.IM_XMPP ),
                        iconCls : QDesk.enums.message.MessageChannel.IM_XMPP
                      },
                      {
                        itemId : 'addImYahooProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.IM_YAHOO ),
                        iconCls : QDesk.enums.message.MessageChannel.IM_YAHOO
                      }
                    ]
                  },

                  /*
                   * SMS message channel profiles.
                   */
                  {
                    text : QDesk.Msg.get(
                        'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.addSmsProfileGroup' ),
                    iconCls : 'addSmsProfile',
                    menu : [
                      {
                        itemId : 'addSmsClickatellCentralProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SMS_CLICKATELL_CENTRAL ),
                        iconCls : QDesk.enums.message.MessageChannel.SMS_CLICKATELL_CENTRAL
                      },
                      {
                        itemId : 'addSmsClickatellPlatformProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SMS_CLICKATELL_PLATFORM ),
                        iconCls : QDesk.enums.message.MessageChannel.SMS_CLICKATELL_PLATFORM
                      },
                      {
                        itemId : 'addSmsClockworkProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SMS_CLOCKWORK ),
                        iconCls : QDesk.enums.message.MessageChannel.SMS_CLOCKWORK
                      },
                      {
                        itemId : 'addSmsNexmoProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SMS_NEXMO ),
                        iconCls : QDesk.enums.message.MessageChannel.SMS_NEXMO
                      },
                      {
                        itemId : 'addSmsSmsGlobalProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SMS_SMSGLOBAL ),
                        iconCls : QDesk.enums.message.MessageChannel.SMS_SMSGLOBAL
                      },
                      {
                        itemId : 'addSmsTwilioProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SMS_TWILIO ),
                        iconCls : QDesk.enums.message.MessageChannel.SMS_TWILIO
                      }
                    ]
                  },

                  /*
                   * SNMP trap message channel profiles.
                   */
                  {
                    text : QDesk.Msg.get(
                        'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.addSnmpProfileGroup' ),
                    iconCls : 'addSnmpProfile',
                    menu : [
                      {
                        itemId : 'addSnmpV1TrapProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SNMPV1_TRAP ),
                        iconCls : QDesk.enums.message.MessageChannel.SNMPV1_TRAP
                      },
                      {
                        itemId : 'addSnmpV2cTrapProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SNMPV2C_TRAP ),
                        iconCls : QDesk.enums.message.MessageChannel.SNMPV2C_TRAP
                      },
                      {
                        itemId : 'addSnmpV3TrapProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName(
                            QDesk.enums.message.MessageChannel.SNMPV3_TRAP ),
                        iconCls : QDesk.enums.message.MessageChannel.SNMPV3_TRAP
                      }
                    ]
                  },

                  /*
                   *  Web service message channel profile.
                   */
                  {
                    itemId : 'addWebServiceProfileButton',
                    text : QDesk.util.EnumUtils.getMessageChannelName(
                        QDesk.enums.message.MessageChannel.WEB_SERVICE ),
                    iconCls : QDesk.enums.message.MessageChannel.WEB_SERVICE
                  }
                ]
              }
            },
            {
              xtype : 'button',
              itemId : 'deleteFolderOrProfileButton',
              scale : 'small',
              iconCls : 'deleteFolderOrProfile',
              // no tooltip, it is set dynamically based on the selection (folder / profile)
              disabled : true
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );
