/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.sms.nexmo.NexmoProfilePanel', {
  extend : 'Ext.form.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.main.message.channel.ProxyPanel',
    'QDesk.view.main.message.channel.sms.nexmo.NexmoBasicPanel'
  ],

  alias : 'widget.SmsNexmoProfilePanel',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3 10 5 10',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'SMS_NEXMO',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get(
                  'QDesk.view.main.message.channel.sms.nexmo.NexmoProfilePanel.title' )
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.main.message.channel.sms.nexmo.NexmoProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.main.message.channel.sms.nexmo.NexmoProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.main.message.channel.sms.nexmo.NexmoProfilePanel.fieldEmptyTxtName' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'NexmoBasicPanel'
            },
            {
              xtype : 'ProxyPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessageEllipsis' ),
          formBind : true
        },
        {
          itemId : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      apiKey : profile.channelConfig.apiKey,
      apiSecret : profile.channelConfig.apiSecret,
      from : profile.channelConfig.from
    } );

    var proxyPanel = me.down( 'ProxyPanel' );
    Ext.apply( values, proxyPanel.proxyConfig2FieldValues( profile.channelConfig.proxy ) );

    // based on the selected proxy type make the proxy host & port form fields required
    proxyPanel.updateProxyFieldsAllowsBlank( profile.channelConfig.proxy );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.enums.message.MessageChannel.SMS_NEXMO );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      apiKey : fieldValues.apiKey,
      apiSecret : fieldValues.apiSecret
    } );

    if ( !Ext.isEmpty( fieldValues.from ) )
    {
      Ext.apply( channelConfig, {
        from : fieldValues.from
      } );
    }

    var proxyPanel = me.down( 'ProxyPanel' );
    var proxy = proxyPanel.fieldValues2ProxyConfig( fieldValues );
    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( channelConfig, {
        proxy : proxy
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
