/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.common.JSScriptingObjectsPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.main.scheduler.common.InfoMessagePanel'
  ],

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  border : false,

  padding : 5,

  alias : 'widget.JSScriptingObjectsPanel',

  cls : 'jsScriptingObjectsPanel',

  qdesk : {
    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    var objectsStore = me._createObjectsTreeStore( config.qdesk.schedulerType );

    Ext.apply( config, {
      items : [
        {
          xtype : 'InfoMessagePanel',
          margin : '0 0 5 0',
          qdesk : {
            msg : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.JSScriptingObjectsPanel.description' )
          }
        },
        {
          xtype : 'treepanel',
          itemId : 'objectsTreePanel',
          border : true,
          layout : 'fit',
          flex : 1,
          cls : 'objectsTreePanel',
          scrollable : true,
          collapsible : false,
          useArrows : true,
          rootVisible : false,
          multiSelect : false,
          singleExpand : false,
          displayField : 'propertyName',
          store : objectsStore,
          columns : [
            {
              xtype : 'treecolumn',
              text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.JSScriptingObjectsPanel.textPropertyName' ),
              flex : 2.7,
              dataIndex : 'propertyName',
              sortable : false,
              renderer : function ( value, metadata, record )
              {
                metadata.style = 'text-decoration: underline';
                return value;
              }
            },
            {
              text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.JSScriptingObjectsPanel.textPropertyType' ),
              flex : 1,
              dataIndex : 'propertyType',
              sortable : false
            },
            {
              text : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.JSScriptingObjectsPanel.textDescription' ),
              flex : 6,
              dataIndex : 'description',
              sortable : false
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  _createObjectsTreeStore : function ( schedulerType )
  {
    var store = null;

    switch ( schedulerType )
    {
      case QDesk.enums.scheduler.SchedulerType.QUARTZ:
        store = Ext.StoreManager.lookup( 'scheduler.quartz.QuartzJSScriptingObjectsTreeStore' );
        break;

      default:
        Ext.Error.raise( 'Unrecognized scheduler type: ' + schedulerType );
        break;
    }

    if ( store == null )
      Ext.Error.raise( 'Objects store for scheduler type: ' + schedulerType + ' not found.' );

    return store;
  }
} );
