/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.common.JSScriptingObjectsWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.scheduler.SchedulerType',
    'QDesk.view.main.scheduler.common.JSScriptingObjectsForm'
  ],

  alias : 'widget.JSScriptingObjectsWindow',

  cls : 'jsScriptingObjectsWindow',
  iconCls : 'windowIcon',

  modal : true,
  resizable : true,

  width : 850,
  height : 500,

  layout : 'fit',

  qdesk : {
    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.scheduler.common.JSScriptingObjectsWindow.windowTitle' ),
      items : [
        {
          xtype : 'JSScriptingObjectsForm',
          itemId : 'form',
          qdesk : {
            schedulerType : config.qdesk.schedulerType
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    me.down( '#closeButton' ).on( 'click', function ()
    {
      me.close();
    } );

    me.down( '#objectsPanel #objectsTreePanel' ).on( 'itemdblclick',
        function ( treePanel, record, item, index, event )
        {
          var propertyName = record.get( 'propertyName' );
          var value = record.getPath( 'propertyName', '.' ).substring( 2 );
          me.fireEvent( 'valueSelected', value );
          me.close();
        }, me );

    me.down( '#dataTypesPanel #dataTypesTreePanel' ).on( 'itemdblclick',
        function ( treePanel, record, item, index, event )
        {
          // double-click on the data type is ignored
          if ( record.getDepth() > 1 )
          {
            var propertyName = record.get( 'propertyName' );

            // remove '..' at the start
            var value = record.getPath( 'propertyName', '.' ).substring( 2 );

            var i = value.indexOf( '.' );
            if ( i > 0 )
            {
              // remove the data type name from the selected value
              value = value.substring( i );
            }
            me.fireEvent( 'valueSelected', value );
            me.close();
          }
        }, me );
  }
} );
