/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.ViewNextFireTimesGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.store.scheduler.quartz.QuartzNextFireTimesStore'
  ],

  alias : 'widget.QuartzViewNextFireTimesGrid',

  cls : 'quartzViewNextFireTimesGrid',

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzNextFireTimesStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.ViewNextFireTimesGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.ViewNextFireTimesGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [ 10, '10' ],
              [ 20, '20' ],
              [ 50, '50' ],
              [ 100, '100' ],
              [ 500, '500' ],
              [ 1000, '1000' ]
              //          [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 70,
          width : 140,
          editable : false,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.ViewNextFireTimesGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ],

      columns : [
        {
          xtype : 'datecolumn',
          itemId : 'nextFireTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.ViewNextFireTimesGrid.textNextFireTimeColumn' ),
          dataIndex : 'value',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          sortable : false,
          groupable : false,
          flex : 1
        }
      ],

      store : store,

      qdesk : {
        gridId : 'quartz.viewNextFireTimesGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [ config ] );

    var maxFetchSize = QDesk.ApplicationConfig.get( me.qdesk.gridId + '.maxFetchSize' );
    me.down( '#maxFetchSizeSelect' ).setValue( maxFetchSize );
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();

    var extra = {};
    extra[ me.qdesk.gridId + '.maxFetchSize' ] = maxFetchSize;

    return extra;
  }
} );
