/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.store.enums.scheduler.quartz.QuartzExecHistoryExportColumnStore',

    'QDesk.view.base.BaseFormPanel',
    // 'QDesk.view.main.scheduler.common.InfoMessagePanel',

    'QDesk.util.constant.DateFormat',

    'Ext.ux.form.ItemSelector'
  ],

  alias : 'widget.QuartzExportExecHistoryForm',

  fieldDefaults : {
    labelWidth : 90
  },

  /* QDesk specific data */
  qdesk : {
    /**
     * Required config parameter with the ID of the Quartz scheduler connection.
     */
    connectionId : undefined,

    /**
     * Optional config parameter containing the job detail whose exec history should be exported and saved.
     */
    jobDetail : undefined,

    /**
     * Optional config parameter containing the trigger whose exec history should be exported and saved.
     */
    trigger : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var exportColumnsStore = Ext.StoreManager.lookup( 'enums.scheduler.quartz.QuartzExecHistoryExportColumnStore' );

    Ext.apply( config, {
      items : [
        // {
        //   xtype : 'InfoMessagePanel',
        //   margin : '0 0 5 0',
        //   qdesk : {
        //     msg : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.description' )
        //   }
        // },
        {
          xtype : 'itemselectorfield',
          itemId : 'selectedColumns',
          name : 'selectedColumns',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldLabelSelectedColumns' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldTooltipSelectedColumns' ),
          labelAlign : 'top',
          allowBlank : false,
          displayField : 'valueLabel',
          valueField : 'value',
          //minSelections : 1,
          buttons : [ 'add', 'remove' ],
          store : exportColumnsStore,
          height : 400
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldsetLabelTimePeriod' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldsetTooltipTimePeriod' ),
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : 'column',
              style : {
                marginBottom : '5px'
              },
              items : [
                /* start date time */
                {
                  xtype : 'xdatetime',
                  itemId : 'startTime',
                  name : 'startTime',
                  allowBlank : false,
                  timeFormat : QDesk.util.constant.DateFormat.timeFormatMedium,
                  dateConfig : {
                    fieldLabel : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldLabelStartTime' ),
                    fieldTooltip : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldTooltipStartTime' ),
                    emptyText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldEmptyTxtStartTimeDate' ),
                    formatText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldFormatTxtStartTimeDate' ),
                    labelWidth : 50,
                    width : 150
                  },
                  timeConfig : {
                    emptyText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldEmptyTxtStartTimeTime' ),
                    formatText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldFormatTxtStartTimeTime' ),
                    increment : 60, /* 60 minutes */
                    width : 105
                  },
                  listeners : {
                    change : {
                      fn : me._onTimePeriodChange,
                      scope : me
                    }
                  }
                },

                {
                  xtype : 'xdatetime',
                  itemId : 'endTime',
                  name : 'endTime',
                  allowBlank : false,
                  timeFormat : QDesk.util.constant.DateFormat.timeFormatMedium,
                  margin : '0 0 0 21',
                  dateConfig : {
                    fieldLabel : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldLabelEndTime' ),
                    fieldTooltip : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldTooltipEndTime' ),
                    emptyText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldEmptyTxtEndTimeDate' ),
                    formatText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldFormatTxtEndTimeDate' ),
                    labelWidth : 37,
                    width : 137
                  },
                  timeConfig : {
                    emptyText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldEmptyTxtEndTimeTime' ),
                    formatText : QDesk.Msg.get(
                        'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldFormatTxtEndTimeTime' ),
                    increment : 60, /* 60 minutes */
                    width : 105
                  },
                  listeners : {
                    change : {
                      fn : me._onTimePeriodChange,
                      scope : me
                    }
                  }
                }
              ]
            }
          ]
        },


        {
          xtype : 'WClearableField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.history.ExportExecHistoryForm.emptyTextFieldFilterQueryText' ),
          labelWidth : 50,
          disabled : false
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'exportButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelExport' ),
          formBind : true
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var now = new Date();

    var todayMidnight = new Date();
    todayMidnight.setTime( now.getTime() );
    todayMidnight.setHours( 0, 0, 0, 0 );

    var tomorrowMidnight = new Date();
    tomorrowMidnight.setTime( now.getTime() );
    tomorrowMidnight.setDate( tomorrowMidnight.getDate() + 1 );
    tomorrowMidnight.setHours( 0, 0, 0, 0 );

    var startTime = me.down( '#startTime' );
    startTime.setValue( todayMidnight );

    var endTime = me.down( '#endTime' );
    endTime.setValue( tomorrowMidnight );
  },


  _onTimePeriodChange : function ( field )
  {
    var me = this;

    var startTime = me.down( '#startTime' );
    var endTime = me.down( '#endTime' );

    if ( startTime.isValid() && endTime.isValid() )
    {
      var startTimeValue = startTime.getValue();
      var endTimeValue = endTime.getValue();

      // startTimeValue <= endTimeValue
      if ( startTimeValue.getTime() <= endTimeValue.getTime() )
      {
        // valid
        return true;
      }
      else
      {
        startTime.markInvalid( 'From datetime must be lower than To end datetime.' );
        endTime.markInvalid( 'From datetime must be lower than To end datetime.' );
        return false;
      }
    }
    else
    {
      return false;
    }
  }
} );
