/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.enums.ProductEdition',
    'QDesk.enums.scheduler.ExecType',

    'QDesk.store.scheduler.quartz.QuartzJobGroupStore',
    'QDesk.store.scheduler.quartz.QuartzExecutingJobStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.quartz.QuartzJobChainUtils',

    'QDesk.view.widget.WSearchField',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzExecutingJobsGrid',

  cls : 'quartzExecutingJobsGrid',

  scrollable : true,

  //border : false,

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textJobDescription\" )]}:</b> ' +
        '<tpl if="jobDetail.description">{[Ext.htmlEncode(values.jobDetail.description)]}</tpl>' +
        '<tpl if="!jobDetail.description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
        '<br/>' +

        '<span style="white-space: nowrap">' +
        '<b>{[QDesk.Msg.get( \"QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textJobMonitoringUrl\" )]}:</b> ' +
        '{[this.getJobMonitoringLink( values.jobDetail.monitoringUrl )]}' +
        '</span>' +
        '</p>' +

        '<br/>' +

        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textTriggerDescription\" )]}:</b> ' +
        '<tpl if="trigger.description">{[Ext.htmlEncode(values.trigger.description)]}</tpl>' +
        '<tpl if="!trigger.description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
        '<br/>' +

        '<span style="white-space: nowrap">' +
        '<b>{[QDesk.Msg.get( \"QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textTriggerMonitoringUrl\" )]}:</b> ' +
        '{[this.getTriggerMonitoringLink( values.trigger.monitoringUrl )]}' +
        '</span>' +
        '</p>',
        {
          getJobMonitoringLink : function ( monitoringUrl )
          {
            var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
            var tooltip;
            if ( licensedFeatures.PollMonitoring )
            {
              tooltip =
                  Ext.htmlEncode( QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobMonitoringUrl' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
            }
            else
            {
              tooltip =
                  Ext.htmlEncode( QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobMonitoringUrlNotLicensed' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                  tooltip + '">' + monitoringUrl + '</a>';
            }
          },
          getTriggerMonitoringLink : function ( monitoringUrl )
          {
            var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
            var tooltip;
            if ( licensedFeatures.PollMonitoring )
            {
              tooltip =
                  Ext.htmlEncode( QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipTriggerMonitoringUrl' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
            }
            else
            {
              tooltip =
                  Ext.htmlEncode( QDesk.Msg.get(
                      'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipTriggerMonitoringUrlNotLicensed' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                  tooltip + '">' + monitoringUrl + '</a>';
            }
          }
        }
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var executingJobStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzExecutingJobStore' );

    executingJobStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      store : executingJobStore,

      qdesk : {
        gridId : 'quartz.executingJobsGrid',
        connectionId : config.qdesk.connectionId,
        jvmAgentInfo : config.qdesk.jvmAgentInfo
      }
    } );

    var jvmAgentReady = config.qdesk.jvmAgentInfo.ready;

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'interruptJobButton',
          scale : 'small',
          iconCls : 'quartzInterruptJob',
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipInterruptJobButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'xcombo',
          itemId : 'filterByJobGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.fieldLabelFilterByJobGroupCombo' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.emptyTextFilterByJobGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true
        },
        //'-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          store : executingJobStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 35,
          width : 400,
          disabled : true,
          hidden : true  /* Index searches are currently not available for the currently executing jobs */
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ],

      columns : [
        // job related columns
        {
          xtype : 'templatecolumn',
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobDetail.name',
          tpl : '{jobDetail.name}',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobDetail.group.name',
          tpl : '{jobDetail.group.name}',
          sortable : true,
          groupable : true,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textJobClassColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobClassColumn' ),
          dataIndex : 'jobDetail.jobClass',
          tpl : '{jobDetail.jobClass}',
          sortable : true,
          groupable : true,
          flex : 1
        },

        // trigger related columns
        {
          xtype : 'templatecolumn',
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'trigger.name',
          tpl : '{trigger.name}',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'trigger.group.name',
          tpl : '{trigger.group.name}',
          sortable : true,
          groupable : true,
          flex : 1
        },

        {
          itemId : 'execTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textExecTypeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeColumn' ),
          dataIndex : 'execType',
          sortable : true,
          groupable : false,
          flex : 0.9,
          renderer : function ( value, metadata, record )
          {
            var text = QDesk.util.EnumUtils.getExecTypeName( value );

            var iconClass, tooltipText;


            if ( value == QDesk.enums.scheduler.ExecType.NORMAL )
            {
              iconClass = 'execTypeNormal';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeNormal' );
            }
            else if ( value == QDesk.enums.scheduler.ExecType.QD_SYSTEM )
            {
              iconClass = 'execTypeQDSystem';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDSystem' );
            }
            else if ( value == QDesk.enums.scheduler.ExecType.QD_MANUAL )
            {
              iconClass = 'execTypeQDManual';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDManual' );
            }
            else if ( value == QDesk.enums.scheduler.ExecType.QD_JOB_CHAIN )
            {
              var jobDataMap = record.get( 'jobDataMap' );
              var jobChainDetails = QDesk.util.quartz.QuartzJobChainUtils.extractJobChainDetails( jobDataMap );

              var jobChainId = jobChainDetails[ 'QD_JOB_CHAIN_ID' ];
              var jobChainName = jobChainDetails[ 'QD_JOB_CHAIN_NAME' ];

              var jobChainSrcJobGroupName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_JOB_GROUP_NAME' ];
              var jobChainSrcJobName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_JOB_NAME' ];

              var jobChainSrcTriggerGroupName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_TRIGGER_GROUP_NAME' ];
              var jobChainSrcTriggerName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_TRIGGER_NAME' ];

              var jobChainSrcExecHistoryId = jobChainDetails[ 'QD_JOB_CHAIN_SRC_EXEC_HISTORY_ID' ];

              var jobChainFlowId = jobChainDetails[ 'QD_JOB_CHAIN_FLOW_ID' ];

              iconClass = 'execTypeQDJobChain';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDJobChain',
                  jobChainId,
                  jobChainName,
                  jobChainSrcJobGroupName,
                  jobChainSrcJobName,
                  jobChainSrcTriggerGroupName,
                  jobChainSrcTriggerName,
                  jobChainSrcExecHistoryId,
                  jobChainFlowId
              );
            }
            else if ( value == QDesk.enums.scheduler.ExecType.QD_WEB_SERVICE )
            {
              iconClass = 'execTypeQDWebService';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipExecTypeQDWebService' );
            }
            else
            {
              Ext.Error.raise( 'Unexpected exec type value: ' + value );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + Ext.htmlEncode( tooltipText ) + '"/>' +
                '<span style="padding-left: 5px;">' + text + '</span>';
          }
        },

        // effective job data map
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textJobDataMapColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var executingJobRecord = grid.getStore().getAt( rowIndex );
                var jobDataMap = executingJobRecord.get( 'jobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap );
              }
            }
          ]
        },

        {
          xtype : 'actioncolumn',
          itemId : 'logColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textLogColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textLogColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipLogColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          items : [
            {
              icon : 'images/icon/scheduler/execution-log-16x16.png',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipLogColumnIcon' ),

              getClass : function ( value, metadata, record )
              {
//                var jobExecutionId = record.get( 'jobExecutionId' );
//                var disabled = Ext.isEmpty( jobExecutionId ) || !jvmAgentReady;
                var disabled = !jvmAgentReady;
                return disabled ? 'nonClickable' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                this.fireEvent( 'requestViewLog', record );
              },
              isDisabled : function ( view, rowIndex, colIndex, item, record )
              {
//                var jobExecutionId = record.get( 'jobExecutionId' );
//                return Ext.isEmpty( jobExecutionId ) || !jvmAgentReady;
                return !jvmAgentReady;
              }
            }
          ]
        },

        // non-jobDetail/non-trigger related columns
        {
          xtype : 'datecolumn',
          itemId : 'fireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textFireTimeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipFireTimeColumn' ),
          dataIndex : 'fireTime',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'datecolumn',
          itemId : 'scheduledFireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textScheduledFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipScheduledFireTimeColumn' ),
          dataIndex : 'scheduledFireTime',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'datecolumn',
          itemId : 'previousFireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textPreviousFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipPreviousFireTimeColumn' ),
          dataIndex : 'previousFireTime',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'datecolumn',
          itemId : 'nextFireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textNextFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipNextFireTimeColumn' ),
          dataIndex : 'nextFireTime',
          sortable : true,
          groupable : true,
          flex : 1.1
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'recoveringColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textRecoveringColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipRecoveringColumn' ),
          dataIndex : 'recovering',
          sortable : false,
          groupable : false,
          width : 80
        },
        {
          xtype : 'templatecolumn',
          itemId : 'refireCountColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textRefireCountColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipRefireCountColumn' ),
          dataIndex : 'refireCount',
          tpl : '{refireCount}',
          sortable : false,
          groupable : false,
          flex : 0.75
        },

        {
          xtype : 'templatecolumn',
          itemId : 'fireInstanceIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.textFireInstanceId' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.tooltipFireInstanceId' ),
          dataIndex : 'fireInstanceId',
          tpl : '{fireInstanceId}',
          sortable : false,
          groupable : false,
          flex : 1
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobGroupStore' );

    jobGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByJobGroupCombo' ).store = jobGroupStore;

    me.getStore().on( 'load', function ( store, records, successful )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.ExecutingJobsGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  }
} );
