/*
 * Copyright (c) 2013-2017 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',
    'QDesk.store.scheduler.quartz.QuartzPlannedJobExecutionStore',

    'QDesk.util.AgentUtils',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',

    'QDesk.util.constant.DateFormat',

    'QDesk.view.widget.WClearableField',
    'QDesk.view.widget.WExecHealthIndicator',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'Ext.ux.form.field.DateTime'
  ],

  alias : 'widget.QuartzPlannedJobExecutionsGrid',

  scrollable : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var schedulerVersion = config.qdesk.schedulerInfo.version;
    var jvmAgentReady = config.qdesk.jvmAgentInfo.ready;

    Ext.apply( config, {
      viewConfig : {
        deferEmptyText : false,
        emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextInitial' )
      }
    } );

    Ext.apply( config, {
      tbar : [
        /* start date time */
        {
          xtype : 'xdatetime',
          itemId : 'startTime',
          name : 'startTime',
          allowBlank : false,
          timeFormat : QDesk.util.constant.DateFormat.timeFormatMedium,
          padding : '0 0 0 3',
          dateConfig : {
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldLabelStartTime' ),
            fieldTooltip : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldTooltipStartTime' ),
            emptyText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldEmptyTxtStartTimeDate' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldFormatTxtStartTimeDate' ),
            labelWidth : 50,
            width : 150
          },
          timeConfig : {
            emptyText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldEmptyTxtStartTimeTime' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldFormatTxtStartTimeTime' ),
            increment : 30, /* 30 minutes */
            width : 105
          },
          listeners : {
            change : {
              fn : me._onTimePeriodChange,
              scope : me
            }
          }
        },

        /* end date time */
        {
          xtype : 'xdatetime',
          itemId : 'endTime',
          name : 'endTime',
          allowBlank : false,
          timeFormat : QDesk.util.constant.DateFormat.timeFormatMedium,
          padding : '0 5 0 5',
          dateConfig : {
            fieldLabel : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldLabelEndTime' ),
            emptyText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldEmptyTxtEndTimeDate' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldFormatTxtEndTimeDate' ),
            labelWidth : 20,
            width : 120
          },
          timeConfig : {
            emptyText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldEmptyTxtEndTimeTime' ),
            formatText : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldFormatTxtEndTimeTime' ),
            increment : 30, /* 30 minutes */
            width : 105
          },
          listeners : {
            change : {
              fn : me._onTimePeriodChange,
              scope : me
            }
          }
        },

        '-',

        {
          xtype : 'WClearableField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextFieldFilterQueryText' ),
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : false
        },

        {
          xtype : 'button',
          itemId : 'submitButton',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.buttonLabelSubmit' ),
          iconCls : 'quartzViewPlannedJobExecutions',
          ui : 'default-small',
          margin : '0 0 0 5'
        },

        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ]
    } )
    ;

    var columns = [];

    columns.push(
        {
          xtype : 'datecolumn',
          itemId : 'fireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipFireTimeColumn' ),
          dataIndex : 'fireTime',
          sortable : false,
          groupable : false,
          flex : 0.8
        },

        {
          xtype : 'templatecolumn',
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobDetail.name',
          tpl : '{jobDetail.name}',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'templatecolumn',
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobDetail.group.name',
          tpl : '{jobDetail.group.name}',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'templatecolumn',
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobClassNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobClassNameColumn' ),
          tpl : '{jobDetail.jobClass}',
          sortable : false,
          groupable : false,
          flex : 1.3
        }
    );

    if ( jvmAgentReady )
    {
      var maxExecHealthIndicatorEntries = QDesk.util.AgentUtils.getJvmAgentConfigProperty(
          config.qdesk.jvmAgentInfo, 'quartz.execHealthIndicators.jobs.maxEntriesPerIndicator' );

      columns.push(
          {
            xtype : 'widgetcolumn',
            itemId : 'jobExecHealthIndicatorColumn',
            text : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobExecHealthIndicatorColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobExecHealthIndicatorColumn',
                maxExecHealthIndicatorEntries ),
            dataIndex : 'jobDetail.execHealthIndicatorValues',
            sortable : false,
            groupable : false,
            width : 115,
            widget : {
              xtype : 'WExecHealthIndicator'
            },
            onWidgetAttach : function ( column, widget, record )
            {
              var jobDetail = record.get( 'jobDetail' );
              if ( jobDetail.execHealthIndicatorValues )
              {
                widget.setValues( jobDetail.execHealthIndicatorValues );
              }
            }
          }
      );
    }

    columns.push(
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobDataMapColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var jobDetail = record.get( 'jobDetail' );
                this.fireEvent( 'requestViewJobDataMap', jobDetail.jobDataMap );
              }
            }
          ]
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobDurabilityColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobDurabilityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobDurabilityColumn' ),
          align : 'center',
          dataIndex : 'jobDetail.durability',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.durability, metaData );  // does not work for nested properties - we use renderer
          }
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobShouldRecoverColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobShouldRecoverColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobShouldRecoverColumn' ),
          align : 'center',
          dataIndex : 'jobDetail.shouldRecover',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.shouldRecover, metaData );  // does not work for nested properties - we use renderer
          }
        },

        // include only for Quartz < 2.x
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobVolatilityColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobVolatilityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobVolatilityColumn' ),
          align : 'center',
          dataIndex : 'jobDetail.volatility',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.volatility, metaData );  // does not work for nested properties - we use renderer
          }
        }
    );

    if ( jvmAgentReady )
    {
      columns.push(
          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'jobConcurrentColumn',
            text : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobConcurrentColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobConcurrentColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.concurrentExecutionDisallowed',
            sortable : false,
            groupable : false,
            width : 90,
            renderer : function ( value, metaData )
            {
              var jobDetail = metaData.record.get( 'jobDetail' );
              return this.defaultRenderer( !jobDetail.concurrentExecutionDisallowed, metaData );  // does not work for nested properties - we use renderer
            }
          },

          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'jobPersistJobDataAfterExecutionColumn',
            text : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobPersistsJobDataColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobPersistsJobDataColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.persistJobDataAfterExecution',
            sortable : false,
            groupable : false,
            width : 120,
            renderer : function ( value, metaData )
            {
              var jobDetail = metaData.record.get( 'jobDetail' );
              return this.defaultRenderer( jobDetail.persistJobDataAfterExecution, metaData );  // does not work for nested properties - we use renderer
            }
          }
      );
    }

    columns.push(
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'triggerName',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'triggerGroupName',
          sortable : false,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'actioncolumn',
          itemId : 'triggerJobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerJobDataMapColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerJobDataMapColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var triggerJobDataMap = record.get( 'triggerJobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', triggerJobDataMap );
              }
            }
          ]
        },

        {
          xtype : 'actioncolumn',
          itemId : 'mergedJobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textMergedJobDataMapColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textMergedJobDataMapColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipMergedJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 150,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipMergedJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var mergedJobDataMap = record.get( 'mergedJobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', mergedJobDataMap );
              }
            }
          ]
        }
    );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzPlannedJobExecutionStore' );

    Ext.apply( config, {
      columns : columns,
      store : store,
      qdesk : {
        gridId : 'quartz.plannedJobExecutionsGrid'
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var now = new Date();

    var todayMidnight = new Date();
    todayMidnight.setTime( now.getTime() );
    todayMidnight.setHours( 0, 0, 0, 0 );

    var tomorrowMidnight = new Date();
    tomorrowMidnight.setTime( now.getTime() );
    tomorrowMidnight.setDate( tomorrowMidnight.getDate() + 1 );
    tomorrowMidnight.setHours( 0, 0, 0, 0 );

    var startTime = me.down( '#startTime' );
    startTime.setValue( todayMidnight );

    var endTime = me.down( '#endTime' );
    endTime.setValue( tomorrowMidnight );

    me.getStore().on( 'load', function ( store, records, successful )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  _onTimePeriodChange : function ( field )
  {
    var me = this;

    var startTime = me.down( '#startTime' );
    var endTime = me.down( '#endTime' );

    var submitButton = me.down( '#submitButton' );
    var filterQueryText = me.down( '#filterQueryText' );

    if ( startTime.isValid() && endTime.isValid() )
    {
      var startTimeValue = startTime.getValue();
      var endTimeValue = endTime.getValue();

      // startTimeValue <= endTimeValue
      if ( startTimeValue.getTime() <= endTimeValue.getTime() )
      {
        filterQueryText.setDisabled( false );
        submitButton.setDisabled( false );
        // valid
        return true;
      }
      else
      {
        startTime.markInvalid( 'From datetime must be lower than To end datetime.' );
        endTime.markInvalid( 'From datetime must be lower than To end datetime.' );
        filterQueryText.setDisabled( true );
        submitButton.setDisabled( true );
        return false;
      }
    }
    else
    {
      filterQueryText.setDisabled( true );
      submitButton.setDisabled( true );
      return false;
    }
  }
} );
